"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const languageModelCache_1 = require("../../embeddedSupport/languageModelCache");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const prettier_1 = require("../../utils/prettier");
const paths_1 = require("../../utils/paths");
const vscode_uri_1 = require("vscode-uri");
const ts = require("typescript");
const _ = require("lodash");
const nullMode_1 = require("../nullMode");
const componentInfo_1 = require("./componentInfo");
// Todo: After upgrading to LS server 4.0, use CompletionContext for filtering trigger chars
// https://microsoft.github.io/language-server-protocol/specification#completion-request-leftwards_arrow_with_hook
const NON_SCRIPT_TRIGGERS = ['<', '*', ':'];
function getJavascriptMode(serviceHost, documentRegions, workspacePath, vueInfoService, dependencyService) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!workspacePath) {
            return Object.assign({}, nullMode_1.nullMode);
        }
        const jsDocuments = languageModelCache_1.getLanguageModelCache(10, 60, document => {
            const vueDocument = documentRegions.refreshAndGet(document);
            return vueDocument.getSingleTypeDocument('script');
        });
        const firstScriptRegion = languageModelCache_1.getLanguageModelCache(10, 60, document => {
            const vueDocument = documentRegions.refreshAndGet(document);
            const scriptRegions = vueDocument.getLanguageRangesOfType('script');
            return scriptRegions.length > 0 ? scriptRegions[0] : undefined;
        });
        let tsModule = ts;
        if (dependencyService) {
            const tsDependency = dependencyService.getDependency('typescript');
            if (tsDependency && tsDependency.state === 0 /* Loaded */) {
                tsModule = tsDependency.module;
            }
        }
        const { updateCurrentVueTextDocument } = serviceHost;
        let config = {};
        let supportedCodeFixCodes;
        return {
            getId() {
                return 'javascript';
            },
            configure(c) {
                config = c;
            },
            updateFileInfo(doc) {
                if (!vueInfoService) {
                    return;
                }
                const { service } = updateCurrentVueTextDocument(doc);
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const info = componentInfo_1.getComponentInfo(tsModule, service, fileFsPath, config);
                if (info) {
                    vueInfoService.updateInfo(doc, info);
                }
            },
            doValidation(doc) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return [];
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const rawScriptDiagnostics = [
                    ...service.getSyntacticDiagnostics(fileFsPath),
                    ...service.getSemanticDiagnostics(fileFsPath)
                ];
                return rawScriptDiagnostics.map(diag => {
                    const tags = [];
                    if (diag.reportsUnnecessary) {
                        tags.push(vscode_languageserver_types_1.DiagnosticTag.Unnecessary);
                    }
                    // syntactic/semantic diagnostic always has start and length
                    // so we can safely cast diag to TextSpan
                    return {
                        range: convertRange(scriptDoc, diag),
                        severity: vscode_languageserver_types_1.DiagnosticSeverity.Error,
                        message: tsModule.flattenDiagnosticMessageText(diag.messageText, '\n'),
                        tags,
                        code: diag.code,
                        source: 'Vetur'
                    };
                });
            },
            doComplete(doc, position) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return { isIncomplete: false, items: [] };
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const offset = scriptDoc.offsetAt(position);
                const triggerChar = doc.getText()[offset - 1];
                if (NON_SCRIPT_TRIGGERS.includes(triggerChar)) {
                    return { isIncomplete: false, items: [] };
                }
                const completions = service.getCompletionsAtPosition(fileFsPath, offset, {
                    includeCompletionsWithInsertText: true,
                    includeCompletionsForModuleExports: _.get(config, ['vetur', 'completion', 'autoImport'])
                });
                if (!completions) {
                    return { isIncomplete: false, items: [] };
                }
                const entries = completions.entries.filter(entry => entry.name !== '__vueEditorBridge');
                return {
                    isIncomplete: false,
                    items: entries.map((entry, index) => {
                        const range = entry.replacementSpan && convertRange(scriptDoc, entry.replacementSpan);
                        const { label, detail } = calculateLabelAndDetailTextForPathImport(entry);
                        return {
                            uri: doc.uri,
                            position,
                            label,
                            detail,
                            sortText: entry.sortText + index,
                            kind: convertKind(entry.kind),
                            textEdit: range && vscode_languageserver_types_1.TextEdit.replace(range, entry.name),
                            data: {
                                // data used for resolving item details (see 'doResolve')
                                languageId: scriptDoc.languageId,
                                uri: doc.uri,
                                offset,
                                source: entry.source
                            }
                        };
                    })
                };
                function calculateLabelAndDetailTextForPathImport(entry) {
                    // Is import path completion
                    if (entry.kind === ts.ScriptElementKind.scriptElement) {
                        if (entry.kindModifiers) {
                            return {
                                label: entry.name,
                                detail: entry.name + entry.kindModifiers
                            };
                        }
                        else {
                            if (entry.name.endsWith('.vue')) {
                                return {
                                    label: entry.name.slice(0, -'.vue'.length),
                                    detail: entry.name
                                };
                            }
                        }
                    }
                    return {
                        label: entry.name,
                        detail: undefined
                    };
                }
            },
            doResolve(doc, item) {
                const { service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return item;
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const details = service.getCompletionEntryDetails(fileFsPath, item.data.offset, item.label, getFormatCodeSettings(config), item.data.source, {
                    importModuleSpecifierEnding: 'minimal',
                    importModuleSpecifierPreference: 'relative',
                    includeCompletionsWithInsertText: true
                });
                if (details && item.kind !== vscode_languageserver_types_1.CompletionItemKind.File && item.kind !== vscode_languageserver_types_1.CompletionItemKind.Folder) {
                    item.detail = tsModule.displayPartsToString(details.displayParts);
                    const documentation = {
                        kind: 'markdown',
                        value: tsModule.displayPartsToString(details.documentation)
                    };
                    if (details.codeActions && config.vetur.completion.autoImport) {
                        const textEdits = convertCodeAction(doc, details.codeActions, firstScriptRegion);
                        item.additionalTextEdits = textEdits;
                        details.codeActions.forEach(action => {
                            if (action.description) {
                                documentation.value += '\n' + action.description;
                            }
                        });
                    }
                    item.documentation = documentation;
                    delete item.data;
                }
                return item;
            },
            doHover(doc, position) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return { contents: [] };
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const info = service.getQuickInfoAtPosition(fileFsPath, scriptDoc.offsetAt(position));
                if (info) {
                    const display = tsModule.displayPartsToString(info.displayParts);
                    const doc = tsModule.displayPartsToString(info.documentation);
                    const markedContents = [{ language: 'ts', value: display }];
                    if (doc) {
                        markedContents.unshift(doc, '\n');
                    }
                    return {
                        range: convertRange(scriptDoc, info.textSpan),
                        contents: markedContents
                    };
                }
                return { contents: [] };
            },
            doSignatureHelp(doc, position) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return nullMode_1.NULL_SIGNATURE;
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const signHelp = service.getSignatureHelpItems(fileFsPath, scriptDoc.offsetAt(position), undefined);
                if (!signHelp) {
                    return nullMode_1.NULL_SIGNATURE;
                }
                const ret = {
                    activeSignature: signHelp.selectedItemIndex,
                    activeParameter: signHelp.argumentIndex,
                    signatures: []
                };
                signHelp.items.forEach(item => {
                    const signature = {
                        label: '',
                        documentation: undefined,
                        parameters: []
                    };
                    signature.label += tsModule.displayPartsToString(item.prefixDisplayParts);
                    item.parameters.forEach((p, i, a) => {
                        const label = tsModule.displayPartsToString(p.displayParts);
                        const parameter = {
                            label,
                            documentation: tsModule.displayPartsToString(p.documentation)
                        };
                        signature.label += label;
                        signature.parameters.push(parameter);
                        if (i < a.length - 1) {
                            signature.label += tsModule.displayPartsToString(item.separatorDisplayParts);
                        }
                    });
                    signature.label += tsModule.displayPartsToString(item.suffixDisplayParts);
                    ret.signatures.push(signature);
                });
                return ret;
            },
            findDocumentHighlight(doc, position) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return [];
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const occurrences = service.getOccurrencesAtPosition(fileFsPath, scriptDoc.offsetAt(position));
                if (occurrences) {
                    return occurrences.map(entry => {
                        return {
                            range: convertRange(scriptDoc, entry.textSpan),
                            kind: entry.isWriteAccess ? vscode_languageserver_types_1.DocumentHighlightKind.Write : vscode_languageserver_types_1.DocumentHighlightKind.Text
                        };
                    });
                }
                return [];
            },
            findDocumentSymbols(doc) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return [];
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const items = service.getNavigationBarItems(fileFsPath);
                if (!items) {
                    return [];
                }
                const result = [];
                const existing = {};
                const collectSymbols = (item, containerLabel) => {
                    const sig = item.text + item.kind + item.spans[0].start;
                    if (item.kind !== 'script' && !existing[sig]) {
                        const symbol = {
                            name: item.text,
                            kind: convertSymbolKind(item.kind),
                            location: {
                                uri: doc.uri,
                                range: convertRange(scriptDoc, item.spans[0])
                            },
                            containerName: containerLabel
                        };
                        existing[sig] = true;
                        result.push(symbol);
                        containerLabel = item.text;
                    }
                    if (item.childItems && item.childItems.length > 0) {
                        for (const child of item.childItems) {
                            collectSymbols(child, containerLabel);
                        }
                    }
                };
                items.forEach(item => collectSymbols(item));
                return result;
            },
            findDefinition(doc, position) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return [];
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const definitions = service.getDefinitionAtPosition(fileFsPath, scriptDoc.offsetAt(position));
                if (!definitions) {
                    return [];
                }
                const definitionResults = [];
                const program = service.getProgram();
                if (!program) {
                    return [];
                }
                definitions.forEach(d => {
                    const definitionTargetDoc = getSourceDoc(d.fileName, program);
                    definitionResults.push({
                        uri: vscode_uri_1.default.file(d.fileName).toString(),
                        range: convertRange(definitionTargetDoc, d.textSpan)
                    });
                });
                return definitionResults;
            },
            findReferences(doc, position) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                if (!languageServiceIncludesFile(service, doc.uri)) {
                    return [];
                }
                const fileFsPath = paths_1.getFileFsPath(doc.uri);
                const references = service.getReferencesAtPosition(fileFsPath, scriptDoc.offsetAt(position));
                if (!references) {
                    return [];
                }
                const referenceResults = [];
                const program = service.getProgram();
                if (!program) {
                    return [];
                }
                references.forEach(r => {
                    const referenceTargetDoc = getSourceDoc(r.fileName, program);
                    if (referenceTargetDoc) {
                        referenceResults.push({
                            uri: vscode_uri_1.default.file(r.fileName).toString(),
                            range: convertRange(referenceTargetDoc, r.textSpan)
                        });
                    }
                });
                return referenceResults;
            },
            getCodeActions(doc, range, _formatParams, context) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                const fileName = paths_1.getFileFsPath(scriptDoc.uri);
                const start = scriptDoc.offsetAt(range.start);
                const end = scriptDoc.offsetAt(range.end);
                if (!supportedCodeFixCodes) {
                    supportedCodeFixCodes = new Set(ts
                        .getSupportedCodeFixes()
                        .map(Number)
                        .filter(x => !isNaN(x)));
                }
                const fixableDiagnosticCodes = context.diagnostics.map(d => +d.code).filter(c => supportedCodeFixCodes.has(c));
                if (!fixableDiagnosticCodes) {
                    return [];
                }
                const formatSettings = getFormatCodeSettings(config);
                const result = [];
                const fixes = service.getCodeFixesAtPosition(fileName, start, end, fixableDiagnosticCodes, formatSettings, 
                /*preferences*/ {});
                collectQuickFixCommands(fixes, service, result);
                const textRange = { pos: start, end };
                const refactorings = service.getApplicableRefactors(fileName, textRange, /*preferences*/ {});
                collectRefactoringCommands(refactorings, fileName, formatSettings, textRange, result);
                return result;
            },
            getRefactorEdits(doc, args) {
                const { service } = updateCurrentVueTextDocument(doc);
                const response = service.getEditsForRefactor(args.fileName, args.formatOptions, args.textRange, args.refactorName, args.actionName, args.preferences);
                if (!response) {
                    // TODO: What happens when there's no response?
                    return createApplyCodeActionCommand('', {});
                }
                const uriMapping = createUriMappingForEdits(response.edits, service);
                return createApplyCodeActionCommand('', uriMapping);
            },
            format(doc, range, formatParams) {
                const { scriptDoc, service } = updateCurrentVueTextDocument(doc);
                const defaultFormatter = scriptDoc.languageId === 'javascript'
                    ? config.vetur.format.defaultFormatter.js
                    : config.vetur.format.defaultFormatter.ts;
                if (defaultFormatter === 'none') {
                    return [];
                }
                const parser = scriptDoc.languageId === 'javascript' ? 'babylon' : 'typescript';
                const needInitialIndent = config.vetur.format.scriptInitialIndent;
                const vlsFormatConfig = config.vetur.format;
                if (defaultFormatter === 'prettier' ||
                    defaultFormatter === 'prettier-eslint' ||
                    defaultFormatter === 'prettier-tslint') {
                    const code = doc.getText(range);
                    const filePath = paths_1.getFileFsPath(scriptDoc.uri);
                    let doFormat;
                    if (defaultFormatter === 'prettier-eslint') {
                        doFormat = prettier_1.prettierEslintify;
                    }
                    else if (defaultFormatter === 'prettier-tslint') {
                        doFormat = prettier_1.prettierTslintify;
                    }
                    else {
                        doFormat = prettier_1.prettierify;
                    }
                    return doFormat(code, filePath, range, vlsFormatConfig, parser, needInitialIndent);
                }
                else {
                    const initialIndentLevel = needInitialIndent ? 1 : 0;
                    const formatSettings = scriptDoc.languageId === 'javascript' ? config.javascript.format : config.typescript.format;
                    const convertedFormatSettings = convertOptions(formatSettings, {
                        tabSize: vlsFormatConfig.options.tabSize,
                        insertSpaces: !vlsFormatConfig.options.useTabs
                    }, initialIndentLevel);
                    const fileFsPath = paths_1.getFileFsPath(doc.uri);
                    const start = scriptDoc.offsetAt(range.start);
                    const end = scriptDoc.offsetAt(range.end);
                    const edits = service.getFormattingEditsForRange(fileFsPath, start, end, convertedFormatSettings);
                    if (!edits) {
                        return [];
                    }
                    const result = [];
                    for (const edit of edits) {
                        if (edit.span.start >= start && edit.span.start + edit.span.length <= end) {
                            result.push({
                                range: convertRange(scriptDoc, edit.span),
                                newText: edit.newText
                            });
                        }
                    }
                    return result;
                }
            },
            onDocumentRemoved(document) {
                jsDocuments.onDocumentRemoved(document);
            },
            onDocumentChanged(filePath) {
                serviceHost.updateExternalDocument(filePath);
            },
            dispose() {
                jsDocuments.dispose();
            }
        };
    });
}
exports.getJavascriptMode = getJavascriptMode;
function collectRefactoringCommands(refactorings, fileName, formatSettings, textRange, result) {
    const actions = [];
    for (const refactoring of refactorings) {
        const refactorName = refactoring.name;
        if (refactoring.inlineable) {
            actions.push({
                fileName,
                formatOptions: formatSettings,
                textRange,
                refactorName,
                actionName: refactorName,
                preferences: {},
                description: refactoring.description
            });
        }
        else {
            actions.push(...refactoring.actions.map(action => ({
                fileName,
                formatOptions: formatSettings,
                textRange,
                refactorName,
                actionName: action.name,
                preferences: {},
                description: action.description
            })));
        }
    }
    for (const action of actions) {
        result.push({
            command: 'vetur.chooseTypeScriptRefactoring',
            title: action.description,
            arguments: [action]
        });
    }
}
function collectQuickFixCommands(fixes, service, result) {
    for (const fix of fixes) {
        const uriTextEditMapping = createUriMappingForEdits(fix.changes, service);
        result.push(createApplyCodeActionCommand(fix.description, uriTextEditMapping));
    }
}
function createApplyCodeActionCommand(title, uriTextEditMapping) {
    return {
        title,
        command: 'vetur.applyWorkspaceEdits',
        arguments: [
            {
                changes: uriTextEditMapping
            }
        ]
    };
}
function createUriMappingForEdits(changes, service) {
    const program = service.getProgram();
    const result = {};
    for (const { fileName, textChanges } of changes) {
        const targetDoc = getSourceDoc(fileName, program);
        const edits = textChanges.map(({ newText, span }) => ({
            newText,
            range: convertRange(targetDoc, span)
        }));
        const uri = vscode_uri_1.default.file(fileName).toString();
        if (result[uri]) {
            result[uri].push(...edits);
        }
        else {
            result[uri] = edits;
        }
    }
    return result;
}
function getSourceDoc(fileName, program) {
    const sourceFile = program.getSourceFile(fileName);
    return vscode_languageserver_types_1.TextDocument.create(fileName, 'vue', 0, sourceFile.getFullText());
}
function languageServiceIncludesFile(ls, documentUri) {
    const filePaths = ls.getProgram().getRootFileNames();
    const filePath = paths_1.getFilePath(documentUri);
    return filePaths.includes(filePath);
}
exports.languageServiceIncludesFile = languageServiceIncludesFile;
function convertRange(document, span) {
    const startPosition = document.positionAt(span.start);
    const endPosition = document.positionAt(span.start + span.length);
    return vscode_languageserver_types_1.Range.create(startPosition, endPosition);
}
function convertKind(kind) {
    switch (kind) {
        case 'primitive type':
        case 'keyword':
            return vscode_languageserver_types_1.CompletionItemKind.Keyword;
        case 'var':
        case 'local var':
            return vscode_languageserver_types_1.CompletionItemKind.Variable;
        case 'property':
        case 'getter':
        case 'setter':
            return vscode_languageserver_types_1.CompletionItemKind.Field;
        case 'function':
        case 'method':
        case 'construct':
        case 'call':
        case 'index':
            return vscode_languageserver_types_1.CompletionItemKind.Function;
        case 'enum':
            return vscode_languageserver_types_1.CompletionItemKind.Enum;
        case 'module':
            return vscode_languageserver_types_1.CompletionItemKind.Module;
        case 'class':
            return vscode_languageserver_types_1.CompletionItemKind.Class;
        case 'interface':
            return vscode_languageserver_types_1.CompletionItemKind.Interface;
        case 'warning':
            return vscode_languageserver_types_1.CompletionItemKind.File;
        case 'script':
            return vscode_languageserver_types_1.CompletionItemKind.File;
        case 'directory':
            return vscode_languageserver_types_1.CompletionItemKind.Folder;
    }
    return vscode_languageserver_types_1.CompletionItemKind.Property;
}
function convertSymbolKind(kind) {
    switch (kind) {
        case 'var':
        case 'local var':
        case 'const':
            return vscode_languageserver_types_1.SymbolKind.Variable;
        case 'function':
        case 'local function':
            return vscode_languageserver_types_1.SymbolKind.Function;
        case 'enum':
            return vscode_languageserver_types_1.SymbolKind.Enum;
        case 'module':
            return vscode_languageserver_types_1.SymbolKind.Module;
        case 'class':
            return vscode_languageserver_types_1.SymbolKind.Class;
        case 'interface':
            return vscode_languageserver_types_1.SymbolKind.Interface;
        case 'method':
            return vscode_languageserver_types_1.SymbolKind.Method;
        case 'property':
        case 'getter':
        case 'setter':
            return vscode_languageserver_types_1.SymbolKind.Property;
    }
    return vscode_languageserver_types_1.SymbolKind.Variable;
}
function convertOptions(formatSettings, options, initialIndentLevel) {
    return _.assign(formatSettings, {
        convertTabsToSpaces: options.insertSpaces,
        tabSize: options.tabSize,
        indentSize: options.tabSize,
        baseIndentSize: options.tabSize * initialIndentLevel
    });
}
function getFormatCodeSettings(config) {
    return {
        tabSize: config.vetur.format.options.tabSize,
        indentSize: config.vetur.format.options.tabSize,
        convertTabsToSpaces: !config.vetur.format.options.useTabs
    };
}
function convertCodeAction(doc, codeActions, regionStart) {
    const scriptStartOffset = doc.offsetAt(regionStart.refreshAndGet(doc).start);
    const textEdits = [];
    for (const action of codeActions) {
        for (const change of action.changes) {
            textEdits.push(...change.textChanges.map(tc => {
                // currently, only import codeAction is available
                // change start of doc to start of script region
                if (tc.span.start <= scriptStartOffset && tc.span.length === 0) {
                    const region = regionStart.refreshAndGet(doc);
                    if (region) {
                        const line = region.start.line;
                        return {
                            range: vscode_languageserver_types_1.Range.create(line + 1, 0, line + 1, 0),
                            newText: tc.newText
                        };
                    }
                }
                return {
                    range: convertRange(doc, tc.span),
                    newText: tc.newText
                };
            }));
        }
    }
    return textEdits;
}
//# sourceMappingURL=javascript.js.map